# *******************************************************************************************
# 
# SharpCap script "UCAC4_annotation.py"
# 2024/04/08 Jean-Francois
#
# The script works only with a PRO version of SharpCap and the ASCOM above 6.5.
#
# Version: 1.0:   Test version
#
#
# The script has to be placed in the list of the Startup script list.
# In SharpCap, please follow these actions:

# - Copy the script somewhere on the computer
# - Start SharpCap
# - In "File" - "SharpCap Settings" - "Startup Scripts" - select this script
# - Close SharpCap (the script will be loaded at the next start)
# - Start SharpCap
# - Connect the camera
# - Connect the telescope (and the focuser if available)
#
# *******************************************************************************************

import os
import math
import clr
clr.AddReference("System.Drawing")
clr.AddReference("System.IO")
clr.AddReference("System.Windows.Forms")
import System.Drawing
import System.Windows.Forms

from math import *
from SharpCap.Interfaces import PlateSolvePurpose
from System import EventHandler
from System.Drawing import *
from System.IO import File
from System.Threading import CancellationToken
from System.Windows.Forms import *


# ****************************************************************************************************************
# To be modified by the user
# ****************************************************************************************************************
UCAC4 = "D:\\Astro_Katalog\\UCAC4\\u4b"                             # Full path location of the UCAC4 star catalog
script_path  = "C:\\Projekte\\Astro\\Scripts_Desktop\\Occultation"  # To be changed if ScharpCap version below 4.1
# ****************************************************************************************************************

Ver_Number       = "UCAC4 stars (1.0)"
SharpCap_version = SharpCap.AppName.Split("v")[1].Split(",")[0]


if (SharpCap_version.CompareTo("4.1") == -1):       # if version below 4.1, path of the script location is needed
    print()
    print("Starting UCAC4 stars script ...")
else:
    print()
    print("Starting UCAC4 stars script ...")
    script_path  = os.path.dirname(__file__)        # if version 4.1 or above, path is found automatically


class OccultationMenuForm(Form):
    def __init__(self):
        self.SuspendLayout()
        self.InitializeComponent()
        self.AutoScaleMode = System.Windows.Forms.AutoScaleMode.Dpi
        self.AutoScaleDimensions = SizeF(96, 96)
        self.ResumeLayout()

    def InitializeComponent(self):
        self.Text = Ver_Number
        self.ClientSize = Size(260, 35)
        self.TopMost = True

        self.buttonUCAC4 = Button()
        self.Controls.Add(self.buttonUCAC4)

        self.buttonUCAC4.Font = Font("Microsoft Sans Serif", 10, FontStyle.Bold)
        self.buttonUCAC4.Location = Point(10, 5)
        self.buttonUCAC4.Name = "buttonUCAC4"
        self.buttonUCAC4.Size = Size(245, 25)
        self.buttonUCAC4.Text = "Copy UCAC4 stars to clipboard"
        self.buttonUCAC4.Click += self.Copy_UCAC4
        self.buttonUCAC4.UseVisualStyleBackColor = True

        if (SharpCap.NightModeControl.Enabled):
            self.BackColor = Color.Black
            self.buttonUCAC4.BackColor = Color.Black
            self.buttonUCAC4.BackColor = Color.Black
            self.buttonUCAC4.ForeColor = Color.DarkOrange
            self.buttonUCAC4.ForeColor = Color.DarkOrange

        if (not SharpCap.NightModeControl.Enabled):
            self.BackColor = Color.White
            self.buttonUCAC4.BackColor = Color.White
            self.buttonUCAC4.BackColor = Color.White
            self.buttonUCAC4.ForeColor = Color.Black
            self.buttonUCAC4.ForeColor = Color.Black

    def Copy_UCAC4(self, sender, args):
        result = SharpCap.SafeGetAsyncResult(SharpCap.BlindSolver.SolveAsync(PlateSolvePurpose.Annotation, CancellationToken()))
        print(result)
        res = SharpCap.SelectedCamera.Controls.Resolution.Value.Split("x")
        p1  = SharpCap.PixelPositionProvider.MapPixel(PointF(0, 0))
        p2  = SharpCap.PixelPositionProvider.MapPixel(PointF(int(res[0]) - 1, 0))
        p3  = SharpCap.PixelPositionProvider.MapPixel(PointF(int(res[0]) - 1, int(res[1]) - 1))
        p4  = SharpCap.PixelPositionProvider.MapPixel(PointF(0, int(res[1]) - 1))
        RA  = [p1.RightAscension, p2.RightAscension, p3.RightAscension, p4.RightAscension]
        DEC = [p1.Declination, p2.Declination, p3.Declination, p4.Declination]
        Delta_RA  = (max(RA) - min(RA)) * 15.0        # Conversion from hours to degrees
        Delta_DEC = max(DEC) - min(DEC)
        if (Delta_RA > 180.0):
            Delta_RA = 360.0 - Delta_RA
        print("Delta_RA = ", Delta_RA)
        print("Delta_DEC = ", Delta_DEC)
        if (Delta_DEC < 0.5):
            Delta_DEC = 0.5
            print("Delta_DEC = ", Delta_DEC)

        annotation_path = os.getenv('APPDATA') + "\\SharpCap\\AnnotationCatalogs"
        command_line = script_path + "\\" + "UCAC4_read " + str(result.RightAscension * 15.0) + " "
        command_line = command_line + str(result.Declination) + " " + str(Delta_RA) + " " + str(Delta_DEC)
        command_line = command_line + " " + UCAC4 + " " + annotation_path
        ret = os.system(command_line)
        print(str(ret) + " stars")

        with open(annotation_path + "\\currentroi.txt", 'r') as f:
            text = f.read()
        f.close()

        System.Windows.Forms.Clipboard.SetText(text)


def launch_form():
    form = OccultationMenuForm()
    form.StartPosition = FormStartPosition.CenterScreen
    form.TopMost = True
    form.Show()

SharpCap.AddCustomButton(" UCAC4  |", None, "Copy UCAC4 stars to clipboard ", launch_form)
